@@ -410,7 +410,7 @@ class ProcessTest(CoverageTest):
         data.read()
         assert line_counts(data)["fork.py"] == total_lines
 
-        debug_text = Path("debug.out").read_text()
+        debug_text = Path("debug.out").read_text(encoding="utf-8")
         ppid = pids["parent"]
         cpid = pids["child"]
         assert ppid != cpid
@@ -666,17 +666,18 @@ class EnvironmentTest(CoverageTest):
         """
         # First, is this even credible try_execfile.py output?
         assert '"DATA": "xyzzy"' in actual
+        # If this fails, "+" is actual, and "-" is expected
         assert actual == expected
 
     def test_coverage_run_is_like_python(self) -> None:
-        with open(TRY_EXECFILE) as f:
+        with open(TRY_EXECFILE, encoding="utf-8") as f:
             self.make_file("run_me.py", f.read())
         expected = self.run_command("python run_me.py")
         actual = self.run_command("coverage run run_me.py")
         self.assert_tryexecfile_output(expected, actual)
 
     def test_coverage_run_far_away_is_like_python(self) -> None:
-        with open(TRY_EXECFILE) as f:
+        with open(TRY_EXECFILE, encoding="utf-8") as f:
             self.make_file("sub/overthere/prog.py", f.read())
         expected = self.run_command("python sub/overthere/prog.py")
         actual = self.run_command("coverage run sub/overthere/prog.py")
@@ -684,7 +685,7 @@ class EnvironmentTest(CoverageTest):
 
     @pytest.mark.skipif(not env.WINDOWS, reason="This is about Windows paths")
     def test_coverage_run_far_away_is_like_python_windows(self) -> None:
-        with open(TRY_EXECFILE) as f:
+        with open(TRY_EXECFILE, encoding="utf-8") as f:
             self.make_file("sub/overthere/prog.py", f.read())
         expected = self.run_command("python sub\\overthere\\prog.py")
         actual = self.run_command("coverage run sub\\overthere\\prog.py")
@@ -697,7 +698,7 @@ class EnvironmentTest(CoverageTest):
         self.assert_tryexecfile_output(expected, actual)
 
     def test_coverage_run_dir_is_like_python_dir(self) -> None:
-        with open(TRY_EXECFILE) as f:
+        with open(TRY_EXECFILE, encoding="utf-8") as f:
             self.make_file("with_main/__main__.py", f.read())
 
         expected = self.run_command("python with_main")
@@ -705,7 +706,7 @@ class EnvironmentTest(CoverageTest):
         self.assert_tryexecfile_output(expected, actual)
 
     def test_coverage_run_dashm_dir_no_init_is_like_python(self) -> None:
-        with open(TRY_EXECFILE) as f:
+        with open(TRY_EXECFILE, encoding="utf-8") as f:
             self.make_file("with_main/__main__.py", f.read())
 
         expected = self.run_command("python -m with_main")
@@ -713,7 +714,7 @@ class EnvironmentTest(CoverageTest):
         self.assert_tryexecfile_output(expected, actual)
 
     def test_coverage_run_dashm_dir_with_init_is_like_python(self) -> None:
-        with open(TRY_EXECFILE) as f:
+        with open(TRY_EXECFILE, encoding="utf-8") as f:
             self.make_file("with_main/__main__.py", f.read())
         self.make_file("with_main/__init__.py", "")
 
@@ -781,7 +782,7 @@ class EnvironmentTest(CoverageTest):
     def test_coverage_run_dashm_is_like_python_dashm_off_path(self) -> None:
         # https://github.com/nedbat/coveragepy/issues/242
         self.make_file("sub/__init__.py", "")
-        with open(TRY_EXECFILE) as f:
+        with open(TRY_EXECFILE, encoding="utf-8") as f:
             self.make_file("sub/run_me.py", f.read())
 
         expected = self.run_command("python -m sub.run_me")
@@ -800,13 +801,46 @@ class EnvironmentTest(CoverageTest):
         # Test running coverage from a zip file itself.  Some environments
         # (windows?) zip up the coverage main to be used as the coverage
         # command.
-        with open(TRY_EXECFILE) as f:
+        with open(TRY_EXECFILE, encoding="utf-8") as f:
             self.make_file("run_me.py", f.read())
         expected = self.run_command("python run_me.py")
         cov_main = os.path.join(TESTS_DIR, "covmain.zip")
         actual = self.run_command(f"python {cov_main} run run_me.py")
         self.assert_tryexecfile_output(expected, actual)
 
+    @pytest.mark.skipif(env.PYVERSION < (3, 11), reason="PYTHONSAFEPATH is new in 3.11")
+    @pytest.mark.skipif(
+        env.WINDOWS,
+        reason="Windows gets this wrong: https://github.com/python/cpython/issues/131484",
+    )
+    def test_pythonsafepath(self) -> None:
+        with open(TRY_EXECFILE, encoding="utf-8") as f:
+            self.make_file("run_me.py", f.read())
+        self.set_environ("PYTHONSAFEPATH", "1")
+        expected = self.run_command("python run_me.py")
+        actual = self.run_command("coverage run run_me.py")
+        self.assert_tryexecfile_output(expected, actual)
+
+    @pytest.mark.skipif(env.PYVERSION < (3, 11), reason="PYTHONSAFEPATH is new in 3.11")
+    def test_pythonsafepath_dashm_runme(self) -> None:
+        with open(TRY_EXECFILE, encoding="utf-8") as f:
+            self.make_file("run_me.py", f.read())
+        self.set_environ("PYTHONSAFEPATH", "1")
+        expected = self.run_command("python run_me.py")
+        actual = self.run_command("python -m coverage run run_me.py")
+        self.assert_tryexecfile_output(expected, actual)
+
+    @pytest.mark.skipif(env.PYVERSION < (3, 11), reason="PYTHONSAFEPATH is new in 3.11")
+    def test_pythonsafepath_dashm(self) -> None:
+        with open(TRY_EXECFILE, encoding="utf-8") as f:
+            self.make_file("with_main/__main__.py", f.read())
+
+        self.set_environ("PYTHONSAFEPATH", "1")
+        expected = self.run_command("python -m with_main")
+        actual = self.run_command("coverage run -m with_main")
+        assert re.search("No module named '?with_main'?", actual)
+        assert re.search("No module named '?with_main'?", expected)
+
     def test_coverage_custom_script(self) -> None:
         # https://github.com/nedbat/coveragepy/issues/678
         # If sys.path[0] isn't the Python default, then coverage.py won't
@@ -1095,7 +1129,6 @@ class CoverageCoreTest(CoverageTest):
         has_ctracer = False
 
     def test_core_default(self) -> None:
-        self.del_environ("COVERAGE_TEST_CORES")
         self.del_environ("COVERAGE_CORE")
         self.make_file("numbers.py", "print(123, 456)")
         out = self.run_command("coverage run --debug=sys numbers.py")
@@ -1103,14 +1136,16 @@ class CoverageCoreTest(CoverageTest):
         core = re_line(r" core:", out).strip()
         # if env.PYBEHAVIOR.pep669:
         #     assert core == "core: SysMonitor"
+        warns = re_lines(r"\(no-ctracer\)", out)
         if self.has_ctracer:
             assert core == "core: CTracer"
+            assert not warns
         else:
             assert core == "core: PyTracer"
+            assert bool(warns) == env.CPYTHON
 
     @pytest.mark.skipif(not has_ctracer, reason="No CTracer to request")
     def test_core_request_ctrace(self) -> None:
-        self.del_environ("COVERAGE_TEST_CORES")
         self.set_environ("COVERAGE_CORE", "ctrace")
         self.make_file("numbers.py", "print(123, 456)")
         out = self.run_command("coverage run --debug=sys numbers.py")
@@ -1119,7 +1154,6 @@ class CoverageCoreTest(CoverageTest):
         assert core == "core: CTracer"
 
     def test_core_request_pytrace(self) -> None:
-        self.del_environ("COVERAGE_TEST_CORES")
         self.set_environ("COVERAGE_CORE", "pytrace")
         self.make_file("numbers.py", "print(123, 456)")
         out = self.run_command("coverage run --debug=sys numbers.py")
@@ -1128,22 +1162,20 @@ class CoverageCoreTest(CoverageTest):
         assert core == "core: PyTracer"
 
     def test_core_request_sysmon(self) -> None:
-        self.del_environ("COVERAGE_TEST_CORES")
         self.set_environ("COVERAGE_CORE", "sysmon")
         self.make_file("numbers.py", "print(123, 456)")
         out = self.run_command("coverage run --debug=sys numbers.py")
         assert out.endswith("123 456\n")
         core = re_line(r" core:", out).strip()
-        warns = re_lines(r"CoverageWarning: sys.monitoring isn't available", out)
+        warns = re_lines(r"\(no-sysmon\)", out)
         if env.PYBEHAVIOR.pep669:
             assert core == "core: SysMonitor"
             assert not warns
         else:
-            assert core in ("core: CTracer", "core: PyTracer")
+            assert core in ["core: CTracer", "core: PyTracer"]
             assert warns
 
     def test_core_request_nosuchcore(self) -> None:
-        self.del_environ("COVERAGE_TEST_CORES")
         self.set_environ("COVERAGE_CORE", "nosuchcore")
         self.make_file("numbers.py", "print(123, 456)")
         out = self.run_command("coverage run numbers.py")
@@ -1222,7 +1254,7 @@ class ProcessStartupTest(CoverageTest):
             """)
         # sub.py will write a few lines.
         self.make_file("sub.py", """\
-            f = open("out.txt", "w")
+            f = open("out.txt", "w", encoding="utf-8")
             f.write("Hello, world!\\n")
             f.close()
             """)
@@ -1242,7 +1274,7 @@ class ProcessStartupTest(CoverageTest):
         self.set_environ("COVERAGE_PROCESS_START", "coverage.ini")
         import main             # pylint: disable=unused-import, import-error
 
-        with open("out.txt") as f:
+        with open("out.txt", encoding="utf-8") as f:
             assert f.read() == "Hello, world!\n"
 
         # Read the data from .coverage
@@ -1261,7 +1293,7 @@ class ProcessStartupTest(CoverageTest):
         self.set_environ("COVERAGE_PROCESS_START", "coverage.ini")
         self.run_command("coverage run main.py")
 
-        with open("out.txt") as f:
+        with open("out.txt", encoding="utf-8") as f:
             assert f.read() == "Hello, world!\n"
 
         self.run_command("coverage combine")
@@ -1334,7 +1366,7 @@ class ProcessStartupWithSourceTest(CoverageTest):
             self.make_file(path("__init__.py"), "")
         # sub.py will write a few lines.
         self.make_file(path("sub.py"), """\
-            f = open("out.txt", "w")
+            f = open("out.txt", "w", encoding="utf-8")
             f.write("Hello, world!")
             f.close()
             """)
@@ -1352,7 +1384,7 @@ class ProcessStartupWithSourceTest(CoverageTest):
 
         self.run_command(cmd)
 
-        with open("out.txt") as f:
+        with open("out.txt", encoding="utf-8") as f:
             assert f.read() == "Hello, world!"
 
         # Read the data from .coverage
